package ru.myitschool.work.data;

import androidx.annotation.NonNull;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

import ru.myitschool.work.data.dto.HistoryDto;
import ru.myitschool.work.data.dto.UserDto;
import ru.myitschool.work.data.network.RetrofitFactory;
import ru.myitschool.work.data.source.CredentialsDataSource;
import ru.myitschool.work.data.source.UserApi;
import ru.myitschool.work.domain.entities.HistoryEntity;
import ru.myitschool.work.domain.entities.Status;
import ru.myitschool.work.domain.entities.UserEntity;

public class UserResponseImpl {

    private static  UserResponseImpl INSTANCE;
    private UserApi userApi = RetrofitFactory.getInstance().getUserApi();
    private final CredentialsDataSource credentialsDataSource = CredentialsDataSource.getInstance();

    private UserResponseImpl() {}

    public static synchronized UserResponseImpl getInstance() {
        if (INSTANCE == null) {
            INSTANCE = new UserResponseImpl();
        }
        return INSTANCE;
    }


    public void getAllUsers(@NonNull Consumer<Status<List<UserEntity>>> callback) {
        userApi.getAllUsers().enqueue(new CallToConsumer<>(
                callback,
                usersDto -> {
                    ArrayList<UserEntity> result = new ArrayList<>(usersDto.size());
                    for (UserDto user : usersDto) {
                        final String username = user.username;
                        final String photo = user.photo;
                        final Long lastVisit = user.lastVisit;
                        final Integer idUser = user.idUser;
                        final String position = user.position;
                        if (idUser != null && username != null) {
                            result.add(new UserEntity(username, photo, lastVisit, idUser, position));
                        }
                    }
                    return result;
                }
        ));

    }

    public void getAllUserHistory(@NonNull Integer id, @NonNull Consumer<Status<List<UserEntity>>> callback) {
        userApi.getAllUserHistory(id).enqueue(new CallToConsumer<>(
                callback,
                historiesDto -> {
                    ArrayList<HistoryEntity> result = new ArrayList<>(historiesDto.size());
                    for (HistoryDto history : historiesDto) {
                        final Integer id1 = history.id;
                        final Integer idUser = history.idUser;
                        final Long time = history.time;
                        final String nameReader = history.nameReader;
                        final String type = history.type;
                        if (idUser != null && id1 != null){
                            result.add(new HistoryEntity(id1, idUser, time, nameReader, type));
                        }
                    }
                    return result;
                }
        ));
    }

    public void login(@NonNull String username, @NonNull String password, Consumer<Status<Void>> callback) {
        credentialsDataSource.updateLogin(username, password);
        userApi = RetrofitFactory.getInstance().getUserApi();
        userApi.login().enqueue(new CallToConsumer<>(
                callback,
                dto -> null
        ));
    }



    public void logout() {
        credentialsDataSource.logout();
    }

}
